@@ -22,14 +22,15 @@ along with this program.  If not, see <http://www.gnu.org/licenses/>.
 
 #include "workspace.h"
 #include "client.h"
+#include "netinfo.h"
 #include "paintredirector.h"
 #include "shadow.h"
 
 namespace KWin
 {
 
-Deleted::Deleted(Workspace* ws)
-    : Toplevel(ws)
+Deleted::Deleted()
+    : Toplevel()
     , delete_refcount(1)
     , no_border(true)
     , padding_left(0)
@@ -38,6 +39,7 @@ Deleted::Deleted(Workspace* ws)
     , padding_bottom(0)
     , m_layer(UnknownLayer)
     , m_minimized(false)
+    , m_modal(false)
     , m_paintRedirector(NULL)
 {
 }
@@ -47,20 +49,20 @@ Deleted::~Deleted()
     if (delete_refcount != 0)
         kError(1212) << "Deleted client has non-zero reference count (" << delete_refcount << ")";
     assert(delete_refcount == 0);
-    workspace()->removeDeleted(this, Allowed);
+    workspace()->removeDeleted(this);
     deleteEffectWindow();
 }
 
 Deleted* Deleted::create(Toplevel* c)
 {
-    Deleted* d = new Deleted(c->workspace());
+    Deleted* d = new Deleted();
     d->copyToDeleted(c);
-    d->workspace()->addDeleted(d, c, Allowed);
+    workspace()->addDeleted(d, c);
     return d;
 }
 
 // to be used only from Workspace::finishCompositing()
-void Deleted::discard(allowed_t)
+void Deleted::discard()
 {
     delete_refcount = 0;
     delete this;
@@ -97,19 +99,23 @@ void Deleted::copyToDeleted(Toplevel* c)
             }
         }
         m_minimized = client->isMinimized();
+        m_modal = client->isModal();
+        m_mainClients = client->mainClients();
+        foreach (Client *c, m_mainClients) {
+            connect(c, SIGNAL(windowClosed(KWin::Toplevel*,KWin::Deleted*)), SLOT(mainClientClosed(KWin::Toplevel*)));
+        }
     }
 }
 
-void Deleted::unrefWindow(bool delay)
+void Deleted::unrefWindow()
 {
     if (--delete_refcount > 0)
         return;
-    // needs to be delayed when calling from effects, otherwise it'd be rather
-    // complicated to handle the case of the window going away during a painting pass
-    if (delay)
-        deleteLater();
-    else
-        delete this;
+    // needs to be delayed
+    // a) when calling from effects, otherwise it'd be rather complicated to handle the case of the
+    // window going away during a painting pass
+    // b) to prevent dangeling pointers in the stacking order, see bug #317765
+    deleteLater();
 }
 
 int Deleted::desktop() const
@@ -170,6 +176,11 @@ NET::WindowType Deleted::windowType(bool direct, int supportedTypes) const
     return info->windowType(supportedTypes);
 }
 
+void Deleted::mainClientClosed(Toplevel *client)
+{
+    m_mainClients.removeAll(static_cast<Client*>(client));
+}
+
 } // namespace
 
 #include "deleted.moc"
