@@ -25,24 +25,30 @@ along with this program.  If not, see <http://www.gnu.org/licenses/>.
 #include "kwineffects.h"
 
 #include "scene.h"
+#include "xcbutils.h"
 
 #include <QStack>
 #include <QHash>
 #include <Plasma/FrameSvg>
 
-
+class QDBusPendingCallWatcher;
+class QDBusServiceWatcher;
 class KService;
+class OrgFreedesktopScreenSaverInterface;
 
 
 namespace KWin
 {
 
-class ThumbnailItem;
+class AbstractThumbnailItem;
+class DesktopThumbnailItem;
+class WindowThumbnailItem;
 
 class Client;
 class Compositor;
 class Deleted;
 class Unmanaged;
+class ScreenLockerWatcher;
 
 class EffectsHandlerImpl : public EffectsHandler
 {
@@ -56,6 +62,10 @@ public:
     virtual ~EffectsHandlerImpl();
     virtual void prePaintScreen(ScreenPrePaintData& data, int time);
     virtual void paintScreen(int mask, QRegion region, ScreenPaintData& data);
+    /**
+     * Special hook to perform a paintScreen but just with the windows on @p desktop.
+     **/
+    void paintDesktop(int desktop, int mask, QRegion region, ScreenPaintData& data);
     virtual void postPaintScreen();
     virtual void prePaintWindow(EffectWindow* w, WindowPrePaintData& data, int time);
     virtual void paintWindow(EffectWindow* w, int mask, QRegion region, WindowPaintData& data);
@@ -100,6 +110,9 @@ public:
     virtual QPoint cursorPos() const;
     virtual bool grabKeyboard(Effect* effect);
     virtual void ungrabKeyboard();
+    // not performing XGrabPointer
+    virtual void startMouseInterception(Effect *effect, Qt::CursorShape shape);
+    virtual void stopMouseInterception(Effect *effect);
     virtual void* getProxy(QString name);
     virtual void startMousePolling();
     virtual void stopMousePolling();
@@ -133,22 +146,20 @@ public:
     virtual double animationTimeFactor() const;
     virtual WindowQuadType newWindowQuadType();
 
-    virtual Window createInputWindow(Effect* e, int x, int y, int w, int h, const QCursor& cursor);
-    using EffectsHandler::createInputWindow;
-    virtual void destroyInputWindow(Window w);
+    virtual void defineCursor(Qt::CursorShape shape);
     virtual bool checkInputWindowEvent(XEvent* e);
     virtual void checkInputWindowStacking();
 
-    virtual void checkElectricBorder(const QPoint &pos, Time time);
-    virtual void reserveElectricBorder(ElectricBorder border);
-    virtual void unreserveElectricBorder(ElectricBorder border);
-    virtual void reserveElectricBorderSwitching(bool reserve, Qt::Orientations o);
+    virtual void reserveElectricBorder(ElectricBorder border, Effect *effect);
+    virtual void unreserveElectricBorder(ElectricBorder border, Effect *effect);
 
     virtual unsigned long xrenderBufferPicture();
     virtual void reconfigure();
     virtual void registerPropertyType(long atom, bool reg);
     virtual QByteArray readRootProperty(long atom, long type, int format) const;
     virtual void deleteRootProperty(long atom) const;
+    virtual xcb_atom_t announceSupportProperty(const QByteArray& propertyName, Effect* effect);
+    virtual void removeSupportProperty(const QByteArray& propertyName, Effect* effect);
 
     virtual bool hasDecorationShadows() const;
 
@@ -159,10 +170,10 @@ public:
     virtual EffectFrame* effectFrame(EffectFrameStyle style, bool staticSize, const QPoint& position, Qt::Alignment alignment) const;
 
     virtual QVariant kwinOption(KWinOption kwopt);
+    virtual bool isScreenLocked() const;
 
     // internal (used by kwin core or compositing code)
     void startPaint();
-    bool borderActivated(ElectricBorder border);
     void grabbedKeyboardEvent(QKeyEvent* e);
     bool hasKeyboardGrab() const;
     void desktopResized(const QSize &size);
@@ -174,12 +185,23 @@ public:
     QList<EffectWindow*> elevatedWindows() const;
     QStringList activeEffects() const;
 
+    /**
+     * @returns Whether we are currently in a desktop rendering process triggered by paintDesktop hook
+     **/
+    bool isDesktopRendering() const {
+        return m_desktopRendering;
+    }
+    /**
+     * @returns the desktop currently being rendered in the paintDesktop hook.
+     **/
+    int currentRenderedDesktop() const {
+        return m_currentRenderedDesktop;
+    }
+
 public Q_SLOTS:
     void slotCurrentTabAboutToChange(EffectWindow* from, EffectWindow* to);
     void slotTabAdded(EffectWindow* from, EffectWindow* to);
     void slotTabRemoved(EffectWindow* c, EffectWindow* newActiveWindow);
-    void slotShowOutline(const QRect &geometry);
-    void slotHideOutline();
 
     // slots for D-Bus interface
     Q_SCRIPTABLE void reconfigureEffect(const QString& name);
@@ -188,6 +210,7 @@ public Q_SLOTS:
     Q_SCRIPTABLE void unloadEffect(const QString& name);
     Q_SCRIPTABLE bool isEffectLoaded(const QString& name) const;
     Q_SCRIPTABLE QString supportInformation(const QString& name) const;
+    Q_SCRIPTABLE QString debug(const QString& name, const QString& parameter = QString()) const;
 
 protected Q_SLOTS:
     void slotDesktopChanged(int old, KWin::Client *withClient);
@@ -223,7 +246,6 @@ protected:
     QMultiMap< int, EffectPair > effect_order;
     QHash< long, int > registered_atoms;
     int next_window_quad_type;
-    int mouse_poll_ref_count;
 
 private Q_SLOTS:
     void slotEffectsQueried();
@@ -235,15 +257,23 @@ private:
     QList< Effect* >::iterator m_currentPaintEffectFrameIterator;
     QList< Effect* >::iterator m_currentPaintScreenIterator;
     QList< Effect* >::iterator m_currentBuildQuadsIterator;
+    typedef QHash< QByteArray, QList< Effect*> > PropertyEffectMap;
+    PropertyEffectMap m_propertiesForEffects;
+    QHash<QByteArray, qulonglong> m_managedProperties;
     Compositor *m_compositor;
     Scene *m_scene;
+    ScreenLockerWatcher *m_screenLockerWatcher;
+    bool m_desktopRendering;
+    int m_currentRenderedDesktop;
+    Xcb::Window m_mouseInterceptionWindow;
+    QList<Effect*> m_grabbedMouseEffects;
 };
 
 class EffectWindowImpl : public EffectWindow
 {
     Q_OBJECT
 public:
-    EffectWindowImpl(Toplevel *toplevel);
+    explicit EffectWindowImpl(Toplevel *toplevel);
     virtual ~EffectWindowImpl();
 
     virtual void enablePainting(int reason);
@@ -265,6 +295,9 @@ public:
 
     virtual WindowQuadList buildQuads(bool force = false) const;
 
+    virtual void referencePreviousWindowPixmap();
+    virtual void unreferencePreviousWindowPixmap();
+
     const Toplevel* window() const;
     Toplevel* window();
 
@@ -273,29 +306,36 @@ public:
     const Scene::Window* sceneWindow() const; // internal
     Scene::Window* sceneWindow(); // internal
 
+    void elevate(bool elevate);
+
     void setData(int role, const QVariant &data);
     QVariant data(int role) const;
 
-    void registerThumbnail(ThumbnailItem *item);
-    QHash<ThumbnailItem*, QWeakPointer<EffectWindowImpl> > const &thumbnails() const {
+    void registerThumbnail(AbstractThumbnailItem *item);
+    QHash<WindowThumbnailItem*, QWeakPointer<EffectWindowImpl> > const &thumbnails() const {
         return m_thumbnails;
     }
+    QList<DesktopThumbnailItem*> const &desktopThumbnails() const {
+        return m_desktopThumbnails;
+    }
 private Q_SLOTS:
     void thumbnailDestroyed(QObject *object);
     void thumbnailTargetChanged();
+    void desktopThumbnailDestroyed(QObject *object);
 private:
-    void insertThumbnail(ThumbnailItem *item);
+    void insertThumbnail(WindowThumbnailItem *item);
     Toplevel* toplevel;
     Scene::Window* sw; // This one is used only during paint pass.
     QHash<int, QVariant> dataMap;
-    QHash<ThumbnailItem*, QWeakPointer<EffectWindowImpl> > m_thumbnails;
+    QHash<WindowThumbnailItem*, QWeakPointer<EffectWindowImpl> > m_thumbnails;
+    QList<DesktopThumbnailItem*> m_desktopThumbnails;
 };
 
 class EffectWindowGroupImpl
     : public EffectWindowGroup
 {
 public:
-    EffectWindowGroupImpl(Group* g);
+    explicit EffectWindowGroupImpl(Group* g);
     virtual EffectWindowList members() const;
 private:
     Group* group;
@@ -382,6 +422,29 @@ private:
     GLShader* m_shader;
 };
 
+class ScreenLockerWatcher : public QObject
+{
+    Q_OBJECT
+public:
+    explicit ScreenLockerWatcher(QObject *parent = 0);
+    virtual ~ScreenLockerWatcher();
+    bool isLocked() const {
+        return m_locked;
+    }
+Q_SIGNALS:
+    void locked(bool locked);
+private Q_SLOTS:
+    void setLocked(bool activated);
+    void activeQueried(QDBusPendingCallWatcher *watcher);
+    void serviceOwnerChanged(const QString &serviceName, const QString &oldOwner, const QString &newOwner);
+    void serviceRegisteredQueried();
+    void serviceOwnerQueried();
+private:
+    OrgFreedesktopScreenSaverInterface *m_interface;
+    QDBusServiceWatcher *m_serviceWatcher;
+    bool m_locked;
+};
+
 inline
 QList<EffectWindow*> EffectsHandlerImpl::elevatedWindows() const
 {
