@@ -2,7 +2,7 @@
  KWin - the KDE window manager
  This file is part of the KDE project.
 
-Copyright (C) 2009 Martin Gräßlin <kde@martin-graesslin.com>
+Copyright (C) 2009 Martin Gräßlin <mgraesslin@kde.org>
 
 This program is free software; you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
@@ -24,8 +24,7 @@ along with this program.  If not, see <http://www.gnu.org/licenses/>.
 
 #include <kwinglutils.h>
 #ifdef KWIN_HAVE_XRENDER_COMPOSITING
-#include <X11/Xlib.h>
-#include <X11/extensions/Xrender.h>
+#include "kwinxrenderutils.h"
 #endif
 
 #include <KColorScheme>
@@ -108,14 +107,14 @@ void ResizeEffect::paintWindow(EffectWindow* w, int mask, QRegion region, Window
 
 #ifdef KWIN_HAVE_XRENDER_COMPOSITING
             if (effects->compositingType() == XRenderCompositing) {
-                XRenderColor col;
-                col.alpha = int(alpha * 0xffff);
-                col.red = int(alpha * 0xffff * color.red() / 255);
-                col.green = int(alpha * 0xffff * color.green() / 255);
-                col.blue = int(alpha * 0xffff * color.blue() / 255);
-                foreach (const QRect & r, paintRegion.rects())
-                XRenderFillRectangle(display(), PictOpOver, effects->xrenderBufferPicture(),
-                                     &col, r.x(), r.y(), r.width(), r.height());
+                QVector<xcb_rectangle_t> rects;
+                foreach (const QRect & r, paintRegion.rects()) {
+                    xcb_rectangle_t rect = {int16_t(r.x()), int16_t(r.y()), uint16_t(r.width()), uint16_t(r.height())};
+                    rects << rect;
+                }
+                xcb_render_fill_rectangles(connection(), XCB_RENDER_PICT_OP_OVER,
+                                           effects->xrenderBufferPicture(), preMultiply(color, alpha),
+                                           rects.count(), rects.constData());
             }
 #endif
         }
