@@ -29,6 +29,8 @@ along with this program.  If not, see <http://www.gnu.org/licenses/>.
 #include <KActionCollection>
 #include <kaction.h>
 #include <KDE/KLocale>
+#include <KDE/KLocalizedString>
+#include <KDE/KGlobal>
 
 using namespace KWin;
 
