@@ -30,11 +30,9 @@ DEALINGS IN THE SOFTWARE.
 #include <QMenu>
 #include <kglobal.h>
 #include <assert.h>
-#if defined Q_WS_X11 && ! defined K_WS_QTONLY
 #include <X11/Xlib.h>
 #include <fixx11h.h>
 #include <QX11Info>
-#endif
 
 #include "kdecorationfactory.h"
 #include "kdecorationbridge.h"
@@ -138,6 +136,11 @@ KDecoration::MaximizeMode KDecoration::maximizeMode() const
     return bridge_->maximizeMode();
 }
 
+KDecoration::QuickTileMode KDecoration::quickTileMode() const
+{
+    return bridge_->quickTileMode();
+}
+
 bool KDecoration::isMinimizable() const
 {
     return bridge_->isMinimizable();
@@ -522,6 +525,11 @@ QRegion KDecoration::region(KDecorationDefines::Region)
     return QRegion();
 }
 
+KDecorationDefines::Position KDecoration::titlebarPosition()
+{
+    return PositionTop;
+}
+
 QString KDecorationDefines::tabDragMimeType()
 {
     return "text/ClientGroupItem";
@@ -614,7 +622,8 @@ KDecorationOptions::BorderSize KDecorationOptions::preferredBorderSize(KDecorati
 
 bool KDecorationOptions::moveResizeMaximizedWindows() const
 {
-    return d->move_resize_maximized_windows;
+    // TODO KF5: remove function with API break
+    return false;
 }
 
 KDecorationDefines::WindowOperation KDecorationOptions::operationMaxButtonClick(Qt::MouseButtons button) const
@@ -660,4 +669,13 @@ void KDecorationOptions::setTitleButtonsRight(const QString& b)
     d->title_buttons_right = b;
 }
 
+extern "C" {
+
+int decoration_bridge_version()
+{
+    return KWIN_DECORATION_BRIDGE_API_VERSION;
+}
+
+}
+
 #include "kdecoration.moc"
