@@ -39,13 +39,18 @@ namespace KWin
 
 #ifndef KCMRULES
 
-Placement::Placement(Workspace* w)
-{
-    m_WorkspacePtr = w;
+KWIN_SINGLETON_FACTORY(Placement)
 
+Placement::Placement(QObject*)
+{
     reinitCascading(0);
 }
 
+Placement::~Placement()
+{
+    s_self = NULL;
+}
+
 /*!
   Places the client \a c according to the workspace's layout policy
  */
@@ -134,6 +139,24 @@ void Placement::placeAtRandom(Client* c, const QRect& area, Policy /*next*/)
     c->move(tx, ty);
 }
 
+// TODO: one day, there'll be C++11 ...
+static inline bool isIrrelevant(Client *client, Client *regarding, int desktop)
+{
+    if (!client)
+        return true;
+    if (client == regarding)
+        return true;
+    if (!client->isCurrentTab())
+        return true;
+    if (!client->isShown(false))
+        return true;
+    if (!client->isOnDesktop(desktop))
+        return true;
+    if (!client->isOnCurrentActivity())
+        return true;
+    return false;
+}
+
 /*!
   Place the client \a c according to a really smart placement algorithm :-)
 */
@@ -152,7 +175,7 @@ void Placement::placeSmart(Client* c, const QRect& area, Policy /*next*/)
     long int overlap, min_overlap = 0;
     int x_optimal, y_optimal;
     int possible;
-    int desktop = c->desktop() == 0 || c->isOnAllDesktops() ? m_WorkspacePtr->currentDesktop() : c->desktop();
+    int desktop = c->desktop() == 0 || c->isOnAllDesktops() ? VirtualDesktopManager::self()->current() : c->desktop();
 
     int cxl, cxr, cyt, cyb;     //temp coords
     int  xl, xr, yt, yb;     //temp coords
@@ -182,29 +205,25 @@ void Placement::placeSmart(Client* c, const QRect& area, Policy /*next*/)
             cxl = x; cxr = x + cw;
             cyt = y; cyb = y + ch;
             ToplevelList::ConstIterator l;
-            for (l = m_WorkspacePtr->stackingOrder().constBegin(); l != m_WorkspacePtr->stackingOrder().constEnd() ; ++l) {
+            for (l = workspace()->stackingOrder().constBegin(); l != workspace()->stackingOrder().constEnd() ; ++l) {
                 Client *client = qobject_cast<Client*>(*l);
-                if (!client) {
+                if (isIrrelevant(client, c, desktop)) {
                     continue;
                 }
-                if (client->isOnDesktop(desktop) &&
-                        client->isShown(false) && client != c) {
+                xl = client->x();          yt = client->y();
+                xr = xl + client->width(); yb = yt + client->height();
 
-                    xl = client->x();          yt = client->y();
-                    xr = xl + client->width(); yb = yt + client->height();
-
-                    //if windows overlap, calc the overall overlapping
-                    if ((cxl < xr) && (cxr > xl) &&
-                            (cyt < yb) && (cyb > yt)) {
-                        xl = qMax(cxl, xl); xr = qMin(cxr, xr);
-                        yt = qMax(cyt, yt); yb = qMin(cyb, yb);
-                        if (client->keepAbove())
-                            overlap += 16 * (xr - xl) * (yb - yt);
-                        else if (client->keepBelow() && !client->isDock()) // ignore KeepBelow windows
-                            overlap += 0; // for placement (see Client::belongsToLayer() for Dock)
-                        else
-                            overlap += (xr - xl) * (yb - yt);
-                    }
+                //if windows overlap, calc the overall overlapping
+                if ((cxl < xr) && (cxr > xl) &&
+                        (cyt < yb) && (cyb > yt)) {
+                    xl = qMax(cxl, xl); xr = qMin(cxr, xr);
+                    yt = qMax(cyt, yt); yb = qMin(cyb, yb);
+                    if (client->keepAbove())
+                        overlap += 16 * (xr - xl) * (yb - yt);
+                    else if (client->keepBelow() && !client->isDock()) // ignore KeepBelow windows
+                        overlap += 0; // for placement (see Client::belongsToLayer() for Dock)
+                    else
+                        overlap += (xr - xl) * (yb - yt);
                 }
             }
         }
@@ -235,27 +254,23 @@ void Placement::placeSmart(Client* c, const QRect& area, Policy /*next*/)
 
             // compare to the position of each client on the same desk
             ToplevelList::ConstIterator l;
-            for (l = m_WorkspacePtr->stackingOrder().constBegin(); l != m_WorkspacePtr->stackingOrder().constEnd() ; ++l) {
+            for (l = workspace()->stackingOrder().constBegin(); l != workspace()->stackingOrder().constEnd() ; ++l) {
                 Client *client = qobject_cast<Client*>(*l);
-                if (!client) {
+                if (isIrrelevant(client, c, desktop)) {
                     continue;
                 }
 
-                if (client->isOnDesktop(desktop) &&
-                        client->isShown(false) && client != c) {
+                xl = client->x();          yt = client->y();
+                xr = xl + client->width(); yb = yt + client->height();
 
-                    xl = client->x();          yt = client->y();
-                    xr = xl + client->width(); yb = yt + client->height();
+                // if not enough room above or under the current tested client
+                // determine the first non-overlapped x position
+                if ((y < yb) && (yt < ch + y)) {
 
-                    // if not enough room above or under the current tested client
-                    // determine the first non-overlapped x position
-                    if ((y < yb) && (yt < ch + y)) {
+                    if ((xr > x) && (possible > xr)) possible = xr;
 
-                        if ((xr > x) && (possible > xr)) possible = xr;
-
-                        basket = xl - cw;
-                        if ((basket > x) && (possible > basket)) possible = basket;
-                    }
+                    basket = xl - cw;
+                    if ((basket > x) && (possible > basket)) possible = basket;
                 }
             }
             x = possible;
@@ -270,24 +285,21 @@ void Placement::placeSmart(Client* c, const QRect& area, Policy /*next*/)
 
             //test the position of each window on the desk
             ToplevelList::ConstIterator l;
-            for (l = m_WorkspacePtr->stackingOrder().constBegin(); l != m_WorkspacePtr->stackingOrder().constEnd() ; ++l) {
+            for (l = workspace()->stackingOrder().constBegin(); l != workspace()->stackingOrder().constEnd() ; ++l) {
                 Client *client = qobject_cast<Client*>(*l);
-                if (!client) {
+                if (isIrrelevant(client, c, desktop)) {
                     continue;
                 }
-                if (client->isOnDesktop(desktop) &&
-                        client != c   &&  c->isShown(false)) {
 
-                    xl = client->x();          yt = client->y();
-                    xr = xl + client->width(); yb = yt + client->height();
+                xl = client->x();          yt = client->y();
+                xr = xl + client->width(); yb = yt + client->height();
 
-                    // if not enough room to the left or right of the current tested client
-                    // determine the first non-overlapped y position
-                    if ((yb > y) && (possible > yb)) possible = yb;
+                // if not enough room to the left or right of the current tested client
+                // determine the first non-overlapped y position
+                if ((yb > y) && (possible > yb)) possible = yb;
 
-                    basket = yt - ch;
-                    if ((basket > y) && (possible > basket)) possible = basket;
-                }
+                basket = yt - ch;
+                if ((basket > y) && (possible > basket)) possible = basket;
             }
             y = possible;
         }
@@ -306,7 +318,7 @@ void Placement::reinitCascading(int desktop)
     // desktop == 0 - reinit all
     if (desktop == 0) {
         cci.clear();
-        for (int i = 0; i < m_WorkspacePtr->numberOfDesktops(); i++) {
+        for (uint i = 0; i < VirtualDesktopManager::self()->count(); ++i) {
             DesktopCascadingInfo inf;
             inf.pos = QPoint(-1, -1);
             inf.col = 0;
@@ -336,9 +348,9 @@ void Placement::placeCascaded(Client* c, QRect& area, Policy nextPlacement)
     int xp, yp;
 
     //CT how do I get from the 'Client' class the size that NW squarish "handle"
-    const QPoint delta = m_WorkspacePtr->cascadeOffset(c);
+    const QPoint delta = workspace()->cascadeOffset(c);
 
-    const int dn = c->desktop() == 0 || c->isOnAllDesktops() ? (m_WorkspacePtr->currentDesktop() - 1) : (c->desktop() - 1);
+    const int dn = c->desktop() == 0 || c->isOnAllDesktops() ? (VirtualDesktopManager::self()->current() - 1) : (c->desktop() - 1);
 
     // get the maximum allowed windows space and desk's origin
     QRect maxRect = checkArea(c, area);
@@ -517,7 +529,7 @@ void Placement::placeMaximizing(Client* c, QRect& area, Policy nextPlacement)
     if (nextPlacement == Unknown)
         nextPlacement = Smart;
     if (c->isMaximizable() && c->maxSize().width() >= area.width() && c->maxSize().height() >= area.height()) {
-        if (m_WorkspacePtr->clientArea(MaximizeArea, c) == area)
+        if (workspace()->clientArea(MaximizeArea, c) == area)
             c->maximize(Client::MaximizeFull);
         else { // if the geometry doesn't match default maximize area (xinerama case?),
             // it's probably better to use the given area
@@ -529,10 +541,44 @@ void Placement::placeMaximizing(Client* c, QRect& area, Policy nextPlacement)
     }
 }
 
+void Placement::cascadeDesktop()
+{
+// TODO XINERAMA this probably is not right for xinerama
+    Workspace *ws = Workspace::self();
+    const int desktop = VirtualDesktopManager::self()->current();
+    reinitCascading(desktop);
+    // TODO: make area const once placeFoo methods are fixed to take a const QRect&
+    QRect area = ws->clientArea(PlacementArea, QPoint(0, 0), desktop);
+    foreach (Toplevel *toplevel, ws->stackingOrder()) {
+        Client *client = qobject_cast<Client*>(toplevel);
+        if (!client ||
+                (!client->isOnCurrentDesktop()) ||
+                (client->isMinimized())         ||
+                (client->isOnAllDesktops())     ||
+                (!client->isMovable()))
+            continue;
+        placeCascaded(client, area);
+    }
+}
+
+void Placement::unclutterDesktop()
+{
+    const ClientList &clients = Workspace::self()->clientList();
+    for (int i = clients.size() - 1; i >= 0; i--) {
+        Client *client = clients.at(i);
+        if ((!client->isOnCurrentDesktop()) ||
+                (client->isMinimized())     ||
+                (client->isOnAllDesktops()) ||
+                (!client->isMovable()))
+            continue;
+        placeSmart(client, QRect());
+    }
+}
+
 QRect Placement::checkArea(const Client* c, const QRect& area)
 {
     if (area.isNull())
-        return m_WorkspacePtr->clientArea(PlacementArea, c->geometry().center(), c->desktop());
+        return workspace()->clientArea(PlacementArea, c->geometry().center(), c->desktop());
     return area;
 }
 
@@ -580,36 +626,47 @@ const char* Placement::policyToString(Policy policy)
 // Workspace
 // ********************
 
+void Client::packTo(int left, int top)
+{
+    const int oldScreen = screen();
+    move(left, top);
+    if (screen() != oldScreen) {
+        workspace()->sendClientToScreen(this, screen()); // checks rule validity
+        if (maximizeMode() != MaximizeRestore)
+            checkWorkspacePosition();
+    }
+}
+
 /*!
   Moves active window left until in bumps into another window or workarea edge.
  */
 void Workspace::slotWindowPackLeft()
 {
     if (active_client && active_client->isMovable())
-        active_client->move(packPositionLeft(active_client, active_client->geometry().left(), true),
-                            active_client->y());
+        active_client->screen();
+        active_client->packTo(packPositionLeft(active_client, active_client->geometry().left(), true),
+                              active_client->y());
 }
 
 void Workspace::slotWindowPackRight()
 {
     if (active_client && active_client->isMovable())
-        active_client->move(
-            packPositionRight(active_client, active_client->geometry().right(), true)
-            - active_client->width() + 1, active_client->y());
+        active_client->packTo(packPositionRight(active_client, active_client->geometry().right(), true)
+                                                - active_client->width() + 1, active_client->y());
 }
 
 void Workspace::slotWindowPackUp()
 {
     if (active_client && active_client->isMovable())
-        active_client->move(active_client->x(),
-                            packPositionUp(active_client, active_client->geometry().top(), true));
+        active_client->packTo(active_client->x(),
+                              packPositionUp(active_client, active_client->geometry().top(), true));
 }
 
 void Workspace::slotWindowPackDown()
 {
     if (active_client && active_client->isMovable())
-        active_client->move(active_client->x(),
-                            packPositionDown(active_client, active_client->geometry().bottom(), true) - active_client->height() + 1);
+        active_client->packTo(active_client->x(),
+                              packPositionDown(active_client, active_client->geometry().bottom(), true) - active_client->height() + 1);
 }
 
 void Workspace::slotWindowGrowHorizontal()
@@ -837,19 +894,6 @@ int Workspace::packPositionDown(const Client* cl, int oldy, bool bottom_edge) co
     return newy;
 }
 
-/*!
-  Asks the internal positioning object to place a client
-*/
-void Workspace::place(Client* c, QRect& area)
-{
-    initPositioning->place(c, area);
-}
-
-void Workspace::placeSmart(Client* c, const QRect& area)
-{
-    initPositioning->placeSmart(c, area);
-}
-
 #endif
 
 } // namespace
