@@ -25,13 +25,14 @@ along with this program.  If not, see <http://www.gnu.org/licenses/>.
 #include <QtDeclarative/qdeclarative.h>
 #include <QtDeclarative/QDeclarativeContext>
 #include <QtDeclarative/QDeclarativeEngine>
-#include <QtGui/QDesktopWidget>
-#include <QtGui/QGraphicsObject>
+#include <QDesktopWidget>
+#include <QGraphicsObject>
 #include <QtGui/QResizeEvent>
 #include <QX11Info>
 
 // include KDE
 #include <KDE/KDebug>
+#include <KDE/KGlobal>
 #include <KDE/KIconEffect>
 #include <KDE/KIconLoader>
 #include <KDE/KServiceTypeTrader>
@@ -64,15 +65,24 @@ QPixmap ImageProvider::requestPixmap(const QString &id, QSize *size, const QSize
     QStringList parts = id.split('/');
     const int row = parts.first().toInt(&ok);
     if (!ok) {
-        return QDeclarativeImageProvider::requestPixmap(id, size, requestedSize);
+        return QPixmap();
     }
-    const QModelIndex index = m_model->index(row, 0);
+    QModelIndex parentIndex;
+    const int parentRow = parts.at(1).toInt(&ok);
+    if (ok) {
+        // we have parent index
+        parentIndex = m_model->index(parentRow, 0);
+        if (!parentIndex.isValid()) {
+            return QPixmap();
+        }
+    }
+    const QModelIndex index = m_model->index(row, 0, parentIndex);
     if (!index.isValid()) {
-        return QDeclarativeImageProvider::requestPixmap(id, size, requestedSize);
+        return QPixmap();
     }
     TabBoxClient* client = static_cast< TabBoxClient* >(index.model()->data(index, ClientModel::ClientRole).value<void *>());
     if (!client) {
-        return QDeclarativeImageProvider::requestPixmap(id, size, requestedSize);
+        return QPixmap();
     }
 
     QSize s(32, 32);
@@ -92,9 +102,9 @@ QPixmap ImageProvider::requestPixmap(const QString &id, QSize *size, const QSize
     if (parts.size() > 2) {
         KIconEffect *effect = KIconLoader::global()->iconEffect();
         KIconLoader::States state = KIconLoader::DefaultState;
-        if (parts.at(2) == QLatin1String("selected")) {
+        if (parts.last() == QLatin1String("selected")) {
             state = KIconLoader::ActiveState;
-        } else if (parts.at(2) == QLatin1String("disabled")) {
+        } else if (parts.last() == QLatin1String("disabled")) {
             state = KIconLoader::DisabledState;
         }
         icon = effect->apply(icon, KIconLoader::Desktop, state);
@@ -102,21 +112,6 @@ QPixmap ImageProvider::requestPixmap(const QString &id, QSize *size, const QSize
     return icon;
 }
 
-// WARNING: this code exists to cover a bug in Qt which prevents plasma from detecting the state change
-// of the compositor through KWindowSystem.
-// once plasma uses (again) a KSelectionWatcher or Qt is fixed in this regard, the code can go.
-static QString plasmaThemeVariant()
-{
-#ifndef TABBOX_KCM
-    if (!Workspace::self()->compositing() || !effects) {
-        return Plasma::Theme::defaultTheme()->currentThemeHasImage("opaque/dialogs/background") ? QLatin1String("opaque/") : QLatin1String("");
-    }
-    if (static_cast<EffectsHandlerImpl*>(effects)->provides(Effect::Blur)) {
-        return Plasma::Theme::defaultTheme()->currentThemeHasImage("translucent/dialogs/background") ? QLatin1String("translucent/") : QLatin1String("");
-    }
-#endif
-    return QLatin1String("");
-}
 
 DeclarativeView::DeclarativeView(QAbstractItemModel *model, TabBoxConfig::TabBoxMode mode, QWidget *parent)
     : QDeclarativeView(parent)
@@ -143,9 +138,11 @@ DeclarativeView::DeclarativeView(QAbstractItemModel *model, TabBoxConfig::TabBox
     kdeclarative.setDeclarativeEngine(engine());
     kdeclarative.initialize();
     kdeclarative.setupBindings();
-    qmlRegisterType<ThumbnailItem>("org.kde.kwin", 0, 1, "ThumbnailItem");
+#ifndef TABBOX_KCM
+    qmlRegisterType<DesktopThumbnailItem>("org.kde.kwin", 0, 1, "DesktopThumbnailItem");
+#endif
+    qmlRegisterType<WindowThumbnailItem>("org.kde.kwin", 0, 1, "ThumbnailItem");
     rootContext()->setContextProperty("viewId", static_cast<qulonglong>(winId()));
-    rootContext()->setContextProperty("plasmaThemeVariant", plasmaThemeVariant());
     if (m_mode == TabBoxConfig::ClientTabBox) {
         rootContext()->setContextProperty("clientModel", model);
     } else if (m_mode == TabBoxConfig::DesktopTabBox) {
@@ -188,7 +185,6 @@ void DeclarativeView::showEvent(QShowEvent *event)
         item->setProperty("currentIndex", tabBox->first().row());
         connect(item, SIGNAL(currentIndexChanged(int)), SLOT(currentIndexChanged(int)));
     }
-    rootContext()->setContextProperty("plasmaThemeVariant", plasmaThemeVariant());
     slotUpdateGeometry();
     QResizeEvent re(size(), size()); // to set mask and blurring.
     resizeEvent(&re);
@@ -338,12 +334,26 @@ void DeclarativeView::updateQmlSource(bool force)
     if (!force && tabBox->config().layoutName() == m_currentLayout) {
         return;
     }
-    if (m_mode == TabBoxConfig::DesktopTabBox) {
-        m_currentLayout = tabBox->config().layoutName();
-        const QString file = KStandardDirs::locate("data", QLatin1String(KWIN_NAME) + QLatin1String("/tabbox/desktop.qml"));
-        rootObject()->setProperty("source", QUrl(file));
+    const bool desktopMode = (m_mode == TabBoxConfig::DesktopTabBox);
+    m_currentLayout = tabBox->config().layoutName();
+    KService::Ptr service = desktopMode ? findDesktopSwitcher() : findWindowSwitcher();
+    if (service.isNull()) {
         return;
     }
+    if (service->property("X-Plasma-API").toString() != "declarativeappletscript") {
+        kDebug(1212) << "Window Switcher Layout is no declarativeappletscript";
+        return;
+    }
+    const QString file = desktopMode ? findDesktopSwitcherScriptFile(service) : findWindowSwitcherScriptFile(service);
+    if (file.isNull()) {
+        kDebug(1212) << "Could not find QML file for window switcher";
+        return;
+    }
+    rootObject()->setProperty("source", QUrl(file));
+}
+
+KService::Ptr DeclarativeView::findWindowSwitcher()
+{
     QString constraint = QString("[X-KDE-PluginInfo-Name] == '%1'").arg(tabBox->config().layoutName());
     KService::List offers = KServiceTypeTrader::self()->query("KWin/WindowSwitcher", constraint);
     if (offers.isEmpty()) {
@@ -352,23 +362,40 @@ void DeclarativeView::updateQmlSource(bool force)
         offers = KServiceTypeTrader::self()->query("KWin/WindowSwitcher", constraint);
         if (offers.isEmpty()) {
             kDebug(1212) << "could not find default window switcher layout";
-            return;
+            return KService::Ptr();
         }
     }
-    m_currentLayout = tabBox->config().layoutName();
-    KService::Ptr service = offers.first();
+    return offers.first();
+}
+
+KService::Ptr DeclarativeView::findDesktopSwitcher()
+{
+    QString constraint = QString("[X-KDE-PluginInfo-Name] == '%1'").arg(tabBox->config().layoutName());
+    KService::List offers = KServiceTypeTrader::self()->query("KWin/DesktopSwitcher", constraint);
+    if (offers.isEmpty()) {
+        // load default
+        constraint = QString("[X-KDE-PluginInfo-Name] == '%1'").arg("informative");
+        offers = KServiceTypeTrader::self()->query("KWin/DesktopSwitcher", constraint);
+        if (offers.isEmpty()) {
+            kDebug(1212) << "could not find default desktop switcher layout";
+            return KService::Ptr();
+        }
+    }
+    return offers.first();
+}
+
+QString DeclarativeView::findWindowSwitcherScriptFile(KService::Ptr service)
+{
     const QString pluginName = service->property("X-KDE-PluginInfo-Name").toString();
-    if (service->property("X-Plasma-API").toString() != "declarativeappletscript") {
-        kDebug(1212) << "Window Switcher Layout is no declarativeappletscript";
-        return;
-    }
     const QString scriptName = service->property("X-Plasma-MainScript").toString();
-    const QString file = KStandardDirs::locate("data", QLatin1String(KWIN_NAME) + "/tabbox/" + pluginName + "/contents/" + scriptName);
-    if (file.isNull()) {
-        kDebug(1212) << "Could not find QML file for window switcher";
-        return;
-    }
-    rootObject()->setProperty("source", QUrl(file));
+    return KStandardDirs::locate("data", QLatin1String(KWIN_NAME) + "/tabbox/" + pluginName + "/contents/" + scriptName);
+}
+
+QString DeclarativeView::findDesktopSwitcherScriptFile(KService::Ptr service)
+{
+    const QString pluginName = service->property("X-KDE-PluginInfo-Name").toString();
+    const QString scriptName = service->property("X-Plasma-MainScript").toString();
+    return KStandardDirs::locate("data", QLatin1String(KWIN_NAME) + "/desktoptabbox/" + pluginName + "/contents/" + scriptName);
 }
 
 void DeclarativeView::slotEmbeddedChanged(bool enabled)
