@@ -580,7 +580,7 @@ void Pager::recalculateWindowRects()
         unsigned long properties[] = { 0, NET::WM2Activities };
         NETWinInfo netInfo(QX11Info::display(), window, QX11Info::appRootWindow(), properties, 2);
         QString result(netInfo.activities());
-        if (!result.isEmpty() && result != "ALL") {
+        if (!result.isEmpty() && result != "00000000-0000-0000-0000-000000000000") {
             QStringList activities = result.split(',');
             if (!activities.contains(m_currentActivity)) {
                 continue;
@@ -799,7 +799,7 @@ void Pager::changeDesktop(int newDesktop)
 
 QPixmap Pager::shadowText(const QString &text)
 {
-    QColor textColor = Plasma::Theme::defaultTheme()->color(Plasma::Theme::TextColor);
+    QColor textColor = m_pagerStyle["textColor"].value<QColor>();
 
     return Plasma::PaintUtils::shadowText(text,
                                           KGlobalSettings::smallestReadableFont(),
