@@ -32,8 +32,6 @@
 #include <Solid/Device>
 #include <Solid/DeviceNotifier>
 
-#include <KAction>
-#include <KActionCollection>
 #include <KDebug>
 #include <KIdleTime>
 #include <KLocalizedString>
@@ -57,6 +55,7 @@ Core::Core(QObject* parent, const KComponentData &componentData)
     , m_applicationData(componentData)
     , m_criticalBatteryTimer(new QTimer(this))
     , m_activityConsumer(new KActivities::Consumer(this))
+    , m_pendingWakeupEvent(true)
 {
 }
 
@@ -121,8 +120,6 @@ void Core::onBackendReady()
         }
     }
 
-    connect(m_backend, SIGNAL(brightnessChanged(float,PowerDevil::BackendInterface::BrightnessControlType)),
-            this, SLOT(onBrightnessChanged(float)));
     connect(m_backend, SIGNAL(acAdapterStateChanged(PowerDevil::BackendInterface::AcAdapterState)),
             this, SLOT(onAcAdapterStateChanged(PowerDevil::BackendInterface::AcAdapterState)));
     connect(m_backend, SIGNAL(batteryRemainingTimeChanged(qulonglong)),
@@ -137,9 +134,6 @@ void Core::onBackendReady()
     // Set up the policy agent
     PowerDevil::PolicyAgent::instance()->init();
 
-    connect(m_backend, SIGNAL(resumeFromSuspend()),
-            this, SLOT(onResumeFromSuspend()));
-
     // Initialize the action pool, which will also load the needed startup actions.
     PowerDevil::ActionPool::instance()->init(this);
 
@@ -154,33 +148,16 @@ void Core::onBackendReady()
     // All systems up Houston, let's go!
     emit coreReady();
     refreshStatus();
+}
 
-    KActionCollection* actionCollection = new KActionCollection( this );
-
-    KAction* globalAction = actionCollection->addAction("Increase Screen Brightness");
-    globalAction->setText(i18nc("Global shortcut", "Increase Screen Brightness"));
-    globalAction->setGlobalShortcut(KShortcut(Qt::Key_MonBrightnessUp));
-    connect(globalAction, SIGNAL(triggered(bool)), SLOT(increaseBrightness()));
-
-    globalAction = actionCollection->addAction("Decrease Screen Brightness");
-    globalAction->setText(i18nc("Global shortcut", "Decrease Screen Brightness"));
-    globalAction->setGlobalShortcut(KShortcut(Qt::Key_MonBrightnessDown));
-    connect(globalAction, SIGNAL(triggered(bool)), SLOT(decreaseBrightness()));
-
-    globalAction = actionCollection->addAction("Sleep");
-    globalAction->setText(i18nc("Global shortcut", "Sleep"));
-    globalAction->setGlobalShortcut(KShortcut(Qt::Key_Sleep));
-    connect(globalAction, SIGNAL(triggered(bool)), SLOT(suspendToRam()));
-
-    globalAction = actionCollection->addAction("Hibernate");
-    globalAction->setText(i18nc("Global shortcut", "Hibernate"));
-    globalAction->setGlobalShortcut(KShortcut(Qt::Key_Hibernate));
-    connect(globalAction, SIGNAL(triggered(bool)), SLOT(suspendToDisk()));
-
-    globalAction = actionCollection->addAction("PowerOff");
-    //globalAction->setText(i18nc("Global shortcut", "Power Off button"));
-    globalAction->setGlobalShortcut(KShortcut(Qt::Key_PowerOff));
-    connect(globalAction, SIGNAL(triggered(bool)), SLOT(powerOffButtonTriggered()));
+bool Core::isActionSupported(const QString& actionName)
+{
+    Action *action = ActionPool::instance()->loadAction(actionName, KConfigGroup(), this);
+    if (!action) {
+        return false;
+    } else {
+        return action->isSupported();
+    }
 }
 
 QString Core::checkBatteryStatus(bool notify)
@@ -387,7 +364,6 @@ void Core::loadProfile(bool force)
         // We are now on a different profile
         m_currentProfile = profileId;
         emit profileChanged(m_currentProfile);
-        emit brightnessChanged(brightness());
     }
 
     // Now... any special behaviors we'd like to consider?
@@ -651,23 +627,6 @@ void Core::onBatteryRemainingTimeChanged(qulonglong time)
     emit batteryRemainingTimeChanged(time);
 }
 
-void Core::onBrightnessChanged(float brightness)
-{
-    emit brightnessChanged(brightness);
-}
-
-void Core::onResumeFromSuspend()
-{
-    // Notify the screensaver
-    OrgFreedesktopScreenSaverInterface iface("org.freedesktop.ScreenSaver",
-                                             "/ScreenSaver",
-                                             QDBusConnection::sessionBus());
-    iface.SimulateUserActivity();
-    PowerDevil::PolicyAgent::instance()->setupSystemdInhibition();
-
-    emit resumingFromSuspend();
-}
-
 void Core::onKIdleTimeoutReached(int identifier, int msec)
 {
     // Find which action(s) requested this idle timeout
@@ -722,36 +681,11 @@ void Core::onResumingFromIdle()
     }
 }
 
-void Core::increaseBrightness()
-{
-    m_backend->brightnessKeyPressed(BackendInterface::Increase);
-}
-
-void Core::decreaseBrightness()
-{
-    m_backend->brightnessKeyPressed(BackendInterface::Decrease);
-}
-
 BackendInterface* Core::backend()
 {
     return m_backend;
 }
 
-void Core::suspendHybrid()
-{
-    triggerSuspendSession(4);
-}
-
-void Core::suspendToDisk()
-{
-    triggerSuspendSession(2);
-}
-
-void Core::suspendToRam()
-{
-    triggerSuspendSession(1);
-}
-
 bool Core::isLidClosed()
 {
     return m_backend->isLidClosed();
@@ -762,40 +696,11 @@ qulonglong Core::batteryRemainingTime() const
     return m_backend->batteryRemainingTime();
 }
 
-int Core::brightness() const
-{
-    return m_backend->brightness();
-}
-
 uint Core::backendCapabilities()
 {
     return m_backend->capabilities();
 }
 
-void Core::setBrightness(int percent)
-{
-    QVariantMap args;
-    args["Value"] = QVariant::fromValue<float>((float)percent);
-    args["Explicit"] = true;
-    ActionPool::instance()->loadAction("BrightnessControl", KConfigGroup(), this)->trigger(args);
-}
-
-void Core::triggerSuspendSession(uint action)
-{
-    PowerDevil::Action *helperAction = ActionPool::instance()->loadAction("SuspendSession", KConfigGroup(), this);
-    if (helperAction) {
-        QVariantMap args;
-        args["Type"] = action;
-        args["Explicit"] = true;
-        helperAction->trigger(args);
-    }
-}
-
-void Core::powerOffButtonTriggered()
-{
-    emit m_backend->buttonPressed(PowerDevil::BackendInterface::PowerButton);
-}
-
 }
 
 #include "powerdevilcore.moc"
